@@ -10,18 +10,21 @@ from ..core.database import DatabaseManager
 from ..core.jobs import JobManager, JobStatus
 from ..utils.debug_log import debug_log, info_logger, error_logger, warning_logger
 
-# New imports for tree-sitter
+# New imports for tree-sitter (using tree-sitter-language-pack)
 from tree_sitter import Language, Parser
-from tree_sitter_languages import get_language
+from ..utils.tree_sitter_manager import get_tree_sitter_manager
 
 class TreeSitterParser:
     """A generic parser wrapper for a specific language using tree-sitter."""
 
     def __init__(self, language_name: str):
         self.language_name = language_name
-        self.language: Language = get_language(language_name)
-        self.parser = Parser()
-        self.parser.set_language(self.language)
+        self.ts_manager = get_tree_sitter_manager()
+        
+        # Get the language (cached) and create a new parser for this instance
+        self.language: Language = self.ts_manager.get_language_safe(language_name)
+        # In tree-sitter 0.25+, Parser takes language in constructor
+        self.parser = Parser(self.language)
 
         self.language_specific_parser = None
         if self.language_name == 'python':
@@ -54,6 +57,13 @@ class TreeSitterParser:
         elif self.language_name == 'c_sharp':
             from .languages.csharp import CSharpTreeSitterParser
             self.language_specific_parser = CSharpTreeSitterParser(self)
+        elif self.language_name == 'php':
+            from .languages.php import PhpTreeSitterParser
+            self.language_specific_parser = PhpTreeSitterParser(self)
+        elif self.language_name == 'kotlin':
+            from .languages.kotlin import KotlinTreeSitterParser
+            self.language_specific_parser = KotlinTreeSitterParser(self)
+
 
 
     def parse(self, file_path: Path, is_dependency: bool = False, **kwargs) -> Dict:
@@ -89,7 +99,11 @@ class GraphBuilder:
             # '.h': TreeSitterParser('c'), # Need to write an algo for distinguishing C vs C++ headers
             '.java': TreeSitterParser('java'),
             '.rb': TreeSitterParser('ruby'),
-            '.cs': TreeSitterParser('c_sharp')
+            '.java': TreeSitterParser('java'),
+            '.rb': TreeSitterParser('ruby'),
+            '.cs': TreeSitterParser('c_sharp'),
+            '.php': TreeSitterParser('php'),
+            '.kt': TreeSitterParser('kotlin'),
         }
         self.create_schema()
 
@@ -147,43 +161,43 @@ class GraphBuilder:
         if '.py' in files_by_lang:
             from .languages import python as python_lang_module
             imports_map.update(python_lang_module.pre_scan_python(files_by_lang['.py'], self.parsers['.py']))
-        elif '.ipynb' in files_by_lang:
+        if '.ipynb' in files_by_lang:
             from .languages import python as python_lang_module
             imports_map.update(python_lang_module.pre_scan_python(files_by_lang['.ipynb'], self.parsers['.ipynb']))
-        elif '.js' in files_by_lang:
+        if '.js' in files_by_lang:
             from .languages import javascript as js_lang_module
             imports_map.update(js_lang_module.pre_scan_javascript(files_by_lang['.js'], self.parsers['.js']))
-        elif '.jsx' in files_by_lang:
+        if '.jsx' in files_by_lang:
             from .languages import javascript as js_lang_module
             imports_map.update(js_lang_module.pre_scan_javascript(files_by_lang['.jsx'], self.parsers['.jsx']))
-        elif '.mjs' in files_by_lang:
+        if '.mjs' in files_by_lang:
             from .languages import javascript as js_lang_module
             imports_map.update(js_lang_module.pre_scan_javascript(files_by_lang['.mjs'], self.parsers['.mjs']))
-        elif '.cjs' in files_by_lang:
+        if '.cjs' in files_by_lang:
             from .languages import javascript as js_lang_module
             imports_map.update(js_lang_module.pre_scan_javascript(files_by_lang['.cjs'], self.parsers['.cjs']))
-        elif '.go' in files_by_lang:
+        if '.go' in files_by_lang:
              from .languages import go as go_lang_module
              imports_map.update(go_lang_module.pre_scan_go(files_by_lang['.go'], self.parsers['.go']))
-        elif '.ts' in files_by_lang:
+        if '.ts' in files_by_lang:
             from .languages import typescript as ts_lang_module
             imports_map.update(ts_lang_module.pre_scan_typescript(files_by_lang['.ts'], self.parsers['.ts']))
-        elif '.tsx' in files_by_lang:
+        if '.tsx' in files_by_lang:
             from .languages import typescript as ts_lang_module
             imports_map.update(ts_lang_module.pre_scan_typescript(files_by_lang['.tsx'], self.parsers['.tsx']))
-        elif '.cpp' in files_by_lang:
+        if '.cpp' in files_by_lang:
             from .languages import cpp as cpp_lang_module
             imports_map.update(cpp_lang_module.pre_scan_cpp(files_by_lang['.cpp'], self.parsers['.cpp']))
-        elif '.h' in files_by_lang:
+        if '.h' in files_by_lang:
             from .languages import cpp as cpp_lang_module
             imports_map.update(cpp_lang_module.pre_scan_cpp(files_by_lang['.h'], self.parsers['.h']))
-        elif '.hpp' in files_by_lang:
+        if '.hpp' in files_by_lang:
             from .languages import cpp as cpp_lang_module
             imports_map.update(cpp_lang_module.pre_scan_cpp(files_by_lang['.hpp'], self.parsers['.hpp']))
-        elif '.rs' in files_by_lang:
+        if '.rs' in files_by_lang:
             from .languages import rust as rust_lang_module
             imports_map.update(rust_lang_module.pre_scan_rust(files_by_lang['.rs'], self.parsers['.rs']))
-        elif '.c' in files_by_lang:
+        if '.c' in files_by_lang:
             from .languages import c as c_lang_module
             imports_map.update(c_lang_module.pre_scan_c(files_by_lang['.c'], self.parsers['.c']))
         elif '.java' in files_by_lang:
@@ -195,6 +209,9 @@ class GraphBuilder:
         elif '.cs' in files_by_lang:
             from .languages import csharp as csharp_lang_module
             imports_map.update(csharp_lang_module.pre_scan_csharp(files_by_lang['.cs'], self.parsers['.cs']))
+        if '.kt' in files_by_lang:
+            from .languages import kotlin as kotlin_lang_module
+            imports_map.update(kotlin_lang_module.pre_scan_kotlin(files_by_lang['.kt'], self.parsers['.kt']))
             
         return imports_map
 
@@ -329,10 +346,12 @@ class GraphBuilder:
                     module_name = imp.get('source')
                     if not module_name: continue
 
-                    # Use a map for relationship properties to handle optional alias
+                    # Use a map for relationship properties to handle optional alias and line_number
                     rel_props = {'imported_name': imp.get('name', '*')}
                     if imp.get('alias'):
                         rel_props['alias'] = imp.get('alias')
+                    if imp.get('line_number'):
+                        rel_props['line_number'] = imp.get('line_number')
 
                     session.run("""
                         MATCH (f:File {path: $file_path})
@@ -347,12 +366,21 @@ class GraphBuilder:
                         set_clauses.append("m.full_import_name = $full_import_name")
                     set_clause_str = ", ".join(set_clauses)
 
+                    # Build relationship properties
+                    rel_props = {}
+                    if imp.get('line_number'):
+                        rel_props['line_number'] = imp.get('line_number')
+                    if imp.get('alias'):
+                        rel_props['alias'] = imp.get('alias')
+
                     session.run(f"""
                         MATCH (f:File {{path: $file_path}})
                         MERGE (m:Module {{name: $name}})
                         SET {set_clause_str}
-                        MERGE (f)-[:IMPORTS]->(m)
-                    """, file_path=file_path_str, **imp)
+                        MERGE (f)-[r:IMPORTS]->(m)
+                        SET r += $rel_props
+                    """, file_path=file_path_str, rel_props=rel_props, **imp)
+
 
             # Handle CONTAINS relationship between class to their children like variables
             for func in file_data.get('functions', []):
@@ -385,7 +413,8 @@ class GraphBuilder:
     def _create_function_calls(self, session, file_data: Dict, imports_map: dict):
         """Create CALLS relationships with a unified, prioritized logic flow for all call types."""
         caller_file_path = str(Path(file_data['file_path']).resolve())
-        local_function_names = {func['name'] for func in file_data.get('functions', [])}
+        local_names = {f['name'] for f in file_data.get('functions', [])} | \
+                      {c['name'] for c in file_data.get('classes', [])}
         local_imports = {imp.get('alias') or imp['name'].split('.')[-1]: imp['name'] 
                         for imp in file_data.get('imports', [])}
         
@@ -394,40 +423,107 @@ class GraphBuilder:
             if called_name in __builtins__: continue
 
             resolved_path = None
+            full_call = call.get('full_name', called_name)
+            base_obj = full_call.split('.')[0] if '.' in full_call else None
             
-            if call.get('inferred_obj_type'):
+            # For chained calls like self.graph_builder.method(), we need to look up 'method'
+            # For direct calls like self.method(), we can use the caller's file
+            is_chained_call = full_call.count('.') > 1 if '.' in full_call else False
+            
+            # Determine the lookup name:
+            # - For chained calls (self.attr.method), use the actual method name
+            # - For direct calls (self.method or module.function), use the base object
+            if is_chained_call and base_obj in ('self', 'this', 'super', 'super()', 'cls', '@'):
+                lookup_name = called_name  # Use the actual method name for lookup
+            else:
+                lookup_name = base_obj if base_obj else called_name
+
+            # 1. Check for local context keywords/direct local names
+            # Only resolve to caller_file_path for DIRECT self/this calls, not chained ones
+            if base_obj in ('self', 'this', 'super', 'super()', 'cls', '@') and not is_chained_call:
+                resolved_path = caller_file_path
+            elif lookup_name in local_names:
+                resolved_path = caller_file_path
+            
+            # 2. Check inferred type if available
+            elif call.get('inferred_obj_type'):
                 obj_type = call['inferred_obj_type']
                 possible_paths = imports_map.get(obj_type, [])
                 if len(possible_paths) > 0:
                     resolved_path = possible_paths[0]
             
-            else:
-                lookup_name = call['full_name'].split('.')[0] if '.' in call['full_name'] else called_name
+            # 3. Check imports map with validation against local imports
+            if not resolved_path:
                 possible_paths = imports_map.get(lookup_name, [])
-
-                if lookup_name in local_function_names:
-                    resolved_path = caller_file_path
-                elif len(possible_paths) == 1:
+                if len(possible_paths) == 1:
                     resolved_path = possible_paths[0]
-                elif len(possible_paths) > 1 and lookup_name in local_imports:
-                    full_import_name = local_imports[lookup_name]
-                    for path in possible_paths:
-                        if full_import_name.replace('.', '/') in path:
-                            resolved_path = path
-                            break
+                elif len(possible_paths) > 1:
+                    if lookup_name in local_imports:
+                        full_import_name = local_imports[lookup_name]
+                        
+                        # Optimization: Check if the FQN is directly in imports_map (from pre-scan)
+                        if full_import_name in imports_map:
+                             direct_paths = imports_map[full_import_name]
+                             if direct_paths and len(direct_paths) == 1:
+                                 resolved_path = direct_paths[0]
+                        
+                        if not resolved_path:
+                            for path in possible_paths:
+                                if full_import_name.replace('.', '/') in path:
+                                    resolved_path = path
+                                    break
             
             if not resolved_path:
-                if called_name in imports_map and imports_map[called_name]:
-                    resolved_path = imports_map[called_name][0]
+                 warning_logger(f"Could not resolve call {called_name} (lookup: {lookup_name}) in {caller_file_path}")
+            # else:
+            #      info_logger(f"Resolved call {called_name} -> {resolved_path}")
+            
+            # Legacy fallback block (was mis-indented)
+            if not resolved_path:
+                possible_paths = imports_map.get(lookup_name, [])
+                if len(possible_paths) > 0:
+                     # Final fallback: global candidate
+                     # Check if it was imported explicitly, otherwise risky
+                     if lookup_name in local_imports:
+                         # We already tried specific matching above, but if we are here
+                         # it means we had ambiguity without matching path?
+                         pass
+                     else:
+                        # Fallback to first available if not imported? Or skip?
+                        # Original logic: resolved_path = possible_paths[0]
+                        # But wait, original code logic was:
+                        pass
+            if not resolved_path:
+                if called_name in local_names:
+                    resolved_path = caller_file_path
+                elif called_name in imports_map and imports_map[called_name]:
+                    # Check if any path in imports_map for called_name matches current file's imports
+                    candidates = imports_map[called_name]
+                    for path in candidates:
+                        for imp_name in local_imports.values():
+                            if imp_name.replace('.', '/') in path:
+                                resolved_path = path
+                                break
+                        if resolved_path: break
+                    if not resolved_path:
+                        resolved_path = candidates[0]
                 else:
                     resolved_path = caller_file_path
 
             caller_context = call.get('context')
             if caller_context and len(caller_context) == 3 and caller_context[0] is not None:
                 caller_name, _, caller_line_number = caller_context
+                # if called_name == "sumOfSquares":
+                    # print(f"DEBUG_CYPHER: caller={caller_name}, caller_line={caller_line_number}, called={called_name}, path={resolved_path}")
+
                 session.run("""
-                    MATCH (caller:Function {name: $caller_name, file_path: $caller_file_path, line_number: $caller_line_number})
-                    MATCH (called:Function {name: $called_name, file_path: $called_file_path})
+                    MATCH (caller) WHERE (caller:Function OR caller:Class) 
+                      AND caller.name = $caller_name 
+                      AND caller.file_path = $caller_file_path 
+                      AND caller.line_number = $caller_line_number
+                    MATCH (called) WHERE (called:Function OR called:Class)
+                      AND called.name = $called_name 
+                      AND called.file_path = $called_file_path
                     MERGE (caller)-[:CALLS {line_number: $line_number, args: $args, full_call_name: $full_call_name}]->(called)
                 """,
                 caller_name=caller_name,
@@ -441,7 +537,9 @@ class GraphBuilder:
             else:
                 session.run("""
                     MATCH (caller:File {path: $caller_file_path})
-                    MATCH (called:Function {name: $called_name, file_path: $called_file_path})
+                    MATCH (called) WHERE (called:Function OR called:Class)
+                      AND called.name = $called_name 
+                      AND called.file_path = $called_file_path
                     MERGE (caller)-[:CALLS {line_number: $line_number, args: $args, full_call_name: $full_call_name}]->(called)
                 """,
                 caller_file_path=caller_file_path,
@@ -707,8 +805,28 @@ class GraphBuilder:
             self.add_repository_to_graph(path, is_dependency)
             repo_name = path.name
 
-            cgcignore_path = path / ".cgcignore"
-            if cgcignore_path.exists():
+            # Search for .cgcignore upwards
+            cgcignore_path = None
+            ignore_root = path.resolve()
+            
+            # Start search from path (or parent if path is file)
+            curr = path.resolve()
+            if not curr.is_dir():
+                curr = curr.parent
+
+            # Walk up looking for .cgcignore
+            while True:
+                candidate = curr / ".cgcignore"
+                if candidate.exists():
+                    cgcignore_path = candidate
+                    ignore_root = curr
+                    debug_log(f"Found .cgcignore at {ignore_root}")
+                    break
+                if curr.parent == curr: # Root hit
+                    break
+                curr = curr.parent
+
+            if cgcignore_path:
                 with open(cgcignore_path) as f:
                     ignore_patterns = f.read().splitlines()
                 spec = pathspec.PathSpec.from_lines('gitwildmatch', ignore_patterns)
@@ -718,8 +836,21 @@ class GraphBuilder:
             supported_extensions = self.parsers.keys()
             all_files = path.rglob("*") if path.is_dir() else [path]
             files = [f for f in all_files if f.is_file() and f.suffix in supported_extensions]
+            
             if spec:
-                files = [f for f in files if not spec.match_file(str(f.relative_to(path)))]
+                filtered_files = []
+                for f in files:
+                    try:
+                        # Match relative to the directory containing .cgcignore
+                        rel_path = f.relative_to(ignore_root)
+                        if not spec.match_file(str(rel_path)):
+                            filtered_files.append(f)
+                        else:
+                            debug_log(f"Ignored file based on .cgcignore: {rel_path}")
+                    except ValueError:
+                        # Should not happen if ignore_root is a parent, but safety fallback
+                        filtered_files.append(f)
+                files = filtered_files
             if job_id:
                 self.job_manager.update_job(job_id, total_files=len(files))
             
